
/*
 Project: Temperature and humidity measurements using DHT11
 MCU: PIC16F886
 Clock: 20.0MHz external crystal
 Board: PIC circuit GLCD board
 Date: Jan 10, 2012
 Written by: Rajendra Bhatt (www.embedded-lab.com)
*/


// Glcd module connections
char GLCD_DataPort at PORTC;

sbit GLCD_CS1 at RB5_bit;
sbit GLCD_CS2 at RB4_bit;
sbit GLCD_RS  at RB0_bit;
sbit GLCD_RW  at RB1_bit;
sbit GLCD_RST  at RB6_bit;
sbit GLCD_EN at RB2_bit;

sbit GLCD_CS1_Direction at TRISB5_bit;
sbit GLCD_CS2_Direction at TRISB4_bit;
sbit GLCD_RS_Direction  at TRISB0_bit;
sbit GLCD_RW_Direction  at TRISB1_bit;
sbit GLCD_RST_Direction  at TRISB6_bit;
sbit GLCD_EN_Direction at TRISB2_bit;
// End Glcd module connections

sbit BackLight at RB3_bit;

sbit Data at RA0_bit;
sbit DataDir at TRISA0_bit;
char message1[] = "Temp = 00.0 C";
char message2[] = "RH   = 00.0 %";
unsigned short TOUT = 0, CheckSum, i;
unsigned short T_Byte1, T_Byte2, RH_Byte1, RH_Byte2;

void StartSignal(){
  DataDir = 0;     // Data port is output
  Data    = 0;
  Delay_ms(25);
  Data    = 1;
  Delay_us(30);
  DataDir = 1;     // Data port is input

}

unsigned short CheckResponse(){
  TOUT = 0;
  TMR2 = 0;
  T2CON.TMR2ON = 1;      // start timer
  while(!Data && !TOUT);
  if (TOUT) return 0;
  else {
   TMR2 = 0;
   while(Data && !TOUT);
   if (TOUT) return 0;
   else {
    T2CON.TMR2ON = 0;
    return 1;
   }
  }
}

unsigned short ReadByte(){
  unsigned short num = 0, t;
  DataDir = 1;
  for (i=0; i<8; i++){
   while(!Data);
   TMR2 = 0;
   T2CON.TMR2ON = 1;
   while(Data);
   T2CON.TMR2ON = 0;
   /*
     For 10.0MHz clock, 1 machine cycle is 0.4 uS. So for 40 us, the
     TMR2 value should be 100
   */
   if(TMR2 > 45) num |= 1<<(7-i);  // If time > 40us, Data is 1
  }
  return num;
}

void interrupt(){
  if(PIR1.TMR2IF){
   TOUT = 1;
   T2CON.TMR2ON = 0; // stop timer
   PIR1.TMR2IF  = 0; // Clear TMR0 interrupt flag
  }
}

void main() {
  unsigned short check;
  ANSEL  = 0;                                      // Configure AN pins as digital
  ANSELH = 0;
  C1ON_bit = 0;                    // Disable comparators
  C2ON_bit = 0;
  TRISB = 0;
  TRISA = 0x01;
  TRISC = 0;                       // configure pins as input

  INTCON.GIE = 1;    //Enable global interrupt
  INTCON.PEIE = 1;   //Enable peripheral interrupt
  // Configure Timer2 module
  PIE1.TMR2IE = 1;  // Enable Timer2 interrupt
  T2CON = 1;        // Prescaler 1:1, and Timer2 is off initially
  PIR1.TMR2IF =0;   // Clear TMR INT Flag bit
  
  TMR2 = 0;
  Glcd_Init();             // Initialize GLCD
  Glcd_Fill(0x00);         // Clear GLCD
  BackLight = 1;

  do {
    Delay_ms(2000);
    StartSignal();
    check = CheckResponse();
    if (!check) {
     Glcd_Fill(0x00);
     Glcd_Set_Font(Font_Glcd_Character8x7, 8, 7, 32);
     Glcd_Write_Text("DHT11 is not", 10, 3, 1);
     Glcd_Write_Text("responding!", 10, 5, 1);
    }
    else{

     RH_Byte1 = ReadByte();
     RH_Byte2 = ReadByte();
     T_Byte1 = ReadByte();
     T_Byte2 = ReadByte();
     CheckSum = ReadByte();
     // Check for error in Data reception
     if (CheckSum == ((RH_Byte1 + RH_Byte2 + T_Byte1 + T_Byte2) & 0xFF))
     {
      message1[7]  = T_Byte1/10 + 48;
      message1[8]  = T_Byte1%10 + 48;
      message1[10] = T_Byte2/10 + 48;
      message2[7]  = RH_Byte1/10 + 48;
      message2[8]  = RH_Byte1%10 + 48;
      message2[10] = RH_Byte2/10 + 48;
      //message1[11] = ' ';     // Degree symbol use 223 if Greek letter appears
      Glcd_Fill(0x00);
      //Glcd_Write_Text(message1, 10, 3, 1);
      Delay_ms(50);
      Glcd_Set_Font(Font_Glcd_Character8x7, 8, 7, 32);
      Glcd_Write_Text(message1, 8, 2, 1);
      Glcd_Write_Text(message2, 8, 4, 1);
      Glcd_Set_Font(Font_Glcd_5x7, 5, 7, 32);
      Glcd_Write_Text("As measured by DHT11", 8, 6, 1);
      }

      else{
       Glcd_Fill(0x00);
       Glcd_Set_Font(Font_Glcd_Character8x7, 8, 7, 32);
       Glcd_Write_Text("Checksum error", 10, 3, 1);
       Glcd_Write_Text("Trying again ...", 10, 4, 1);

      }
    }

  }while(1);
}